import React, {useEffect} from 'react'
import axios from 'axios'
import semver from 'semver'
import ExecutionEnvironment from '@docusaurus/ExecutionEnvironment'
import useDocusaurusContext from '@docusaurus/useDocusaurusContext'
import BrowserOnly from '@docusaurus/BrowserOnly'

import useAsync from '@site/src/hooks/useAsync'

const SCHEMA_BASE_URL = 'https://docs.fitko.de'

const getLatestVersion = async (siteVersion) => {
  return axios.get('https://git.fitko.de/api/v4/projects/1/repository/tags')
    .catch((error) => {
      throw `Fetching of latest tags failed with ${error.response.status}`
    })
    .then(async ({ data }) => {
      if (siteVersion === 'next') {
        return data[0].name
      } else {
        const versionRange = `~${siteVersion}`
        const suitableVersions = await data
          .filter(({ name }) => semver.satisfies(name, versionRange))
          .map(({ name }) => name)
          .sort()
          .reverse()
        console.log('suitableVersions', suitableVersions, suitableVersions[0])
        return suitableVersions[0]
      }
    })
}

const DownloadLabel = ({ version, artifact }) => {
  return <a href={`${SCHEMA_BASE_URL}/${version}/${artifact}`} download>
    (herunterladen)
  </a>
}

export default function ApiSpec(props) {
  const isInBrowser = ExecutionEnvironment.canUseDOM
  const { siteVersion = 'next' } = useDocusaurusContext().siteMetadata

  const { execute, status, error, value: latestVersion } = useAsync(getLatestVersion, [siteVersion], false)

  if (isInBrowser) {
    useEffect(() => {
      execute()
    }, [])
  }

  return (
    <BrowserOnly fallback={<div>Lädt...</div>}>
      {() => {
        import('rapidoc')

        return (
          <div>
            <p>Die aktuell angezeigte Version der API ist {status === 'success' && <code>{latestVersion}</code>} {status === 'success' && <DownloadLabel version={latestVersion} artifact={props.artifact} />}.</p>

            { status === 'success' && latestVersion && <rapi-doc
              render-style="view"
              layout="column"
              spec-url={`${SCHEMA_BASE_URL}/${latestVersion}/${props.artifact}`}
              theme="light"
              show-info="false"
              show-header="false"
              show-components="false"
              allow-spec-file-load="false"
              info-description-headings-in-navbar="false"
              allow-try="false"
              primary-color="#11171a"
              allow-server-selection="false"
              server-url=""
            />}

            {status === 'error' && <p>Die API-Spezifikation konnte leider nicht geladen werden.</p>}
          </div>
        )
      }}
    </BrowserOnly>
  )
}
